#!/usr/bin/env python
# -*- coding: utf-8 -*-
class HubInfo:
	
	def __init__(self):
		self.hubsupports = []
		self.hubname = ""
		self.topic = ""
		self.hubversion = ""
		self.info = ""
		self.lastmessage = ""
		self.tag = None
